@@ -380,7 +380,7 @@ class ModelFactory():
 
         '''
         if len(eigen_path) == 0:
-            eigen_path = self.eigen_paths[0]
+            eigen_path = self.eigen_path[0]
 
         #eigen_path_str = r'{}'.format(eigen_path) #convert to raw string
 
